SystemOrganization addCategory: #'Smallchange-Core'!
SystemOrganization addCategory: #'Smallchange-Core-Development'!
SystemOrganization addCategory: #'Smallchange-Core-Announcements'!
SystemOrganization addCategory: #'Smallchange-Core-Commands'!
SystemOrganization addCategory: #'Smallchange-Core-Analysers'!

MbObject subclass: #ScObject
	instanceVariableNames: 'profile announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

ScObject subclass: #ScAnalyser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAnalyser commentStamp: 'StuartHerring 3/13/2012 19:55' prior: 0!
My subclasses are responsible for analysing a profile and reporting on certain conditions!

ScAnalyser subclass: #ScAccountAnalyserGroup
	instanceVariableNames: 'analysers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAccountAnalyserGroup commentStamp: 'StuartHerring 6/13/2012 17:46' prior: 0!
ScAccountAnalyserGroup 
	An Analyser that performs a single analysis over each account in a profile.!

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 20:49'!
accounts
	^#()! !

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 18:28'!
analyserClass
	self subclassResponsibility! !

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:47'!
balanceChanged: anAccount
	analysers do: [ :a | a balanceChanged: anAccount ]! !

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/17/2012 20:09'!
initializeWithProfile: aProfile
	super initializeWithProfile: aProfile.
	analysers := self accounts 
		inject: (OrderedCollection new) 
		into: [ :group :a | 
			group 
				add: (self analyserClass profile: profile account: a); 
				yourself ]! !

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/17/2012 20:09'!
notices
	^analysers gather: [ :each | each notices ]! !

!ScAccountAnalyserGroup methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:20'!
registerSubscriptionsOn: anAnnouncer
	super registerSubscriptionsOn: anAnnouncer.
	anAnnouncer on: ScBalanceChanged do: [ :a | self balanceChanged: a account ].! !

ScAccountAnalyserGroup subclass: #ScAccountsNotOverdrawn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAccountsNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 20:49'!
accounts
	^profile accounts! !

!ScAccountsNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 18:30'!
analyserClass
	^ScAccountNotOverdrawn ! !

ScAccountAnalyserGroup subclass: #ScPoolsNotOverdrawn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScPoolsNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 20:50'!
accounts
	^profile pools! !

!ScPoolsNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 18:44'!
analyserClass
	^ScAccountNotOverdrawn ! !

ScAnalyser subclass: #ScAccountNotOverdrawn
	instanceVariableNames: 'account notices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAccountNotOverdrawn class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 18:31'!
profile: aProfile account: anAccount
	^self basicNew 
		initializeWithProfile: aProfile account: anAccount;
		yourself.! !

!ScAccountNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:46'!
balanceChanged: anAccount
	account = anAccount ifTrue: [ self updateNotices]! !

!ScAccountNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:24'!
initializeWithProfile: aProfile account: anAccount
	self initializeWithProfile: aProfile.
	account := anAccount.
	notices := nil.
	self modificationAnnouncer: anAccount modificationAnnouncer
	! !

!ScAccountNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 19:12'!
notices
	^notices ifNil: [ self updateNotices. notices ]! !

!ScAccountNotOverdrawn methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:47'!
updateNotices
	notices := (account balance roundTo: 1 / 100) < 0 
		ifTrue: [ Array with: (ScNegativeBalanceNotice account: account)]
		ifFalse: [ #() ]! !

ScAnalyser subclass: #ScAccountsSufficientForBills
	instanceVariableNames: 'notices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAccountsSufficientForBills methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 19:02'!
addNoticeFor: aTransactionHolder inSchedule: aSchedule shortBy: anAmount
	| n |
	n := ScInsufficientFundsNotice new
		schedule: aSchedule;
		amount: anAmount.
	aTransactionHolder isAccount ifTrue: [ n account: aTransactionHolder].
	aTransactionHolder isPool ifTrue:  [n pool: aTransactionHolder ].
	notices add: n! !

!ScAccountsSufficientForBills methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 19:41'!
notices
	^notices ifNil: [ self updateNotices. notices ]! !

!ScAccountsSufficientForBills methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:17'!
registerSubscriptionsOn: anAnnouncer
	anAnnouncer
		on: ScBalanceChanged, ScScheduleRemoved, ScScheduleAdded, ScScheduleModified send: #updateNotices to: self! !

!ScAccountsSufficientForBills methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:59'!
testBill: b
	| account amounts pool |
	account := b sourceAccount.
	pool := b sourcePool ifNil: [ profile unallocated ].
	amounts := (profile schedules gather: [ :s | s transactionsBetween: Date today and: b nextDate ]) 
			inject: account balance @ pool balance
			into: [ :balances :transaction | 
				(balances x + (account accumulate: transaction)) @
				(balances y + (pool accumulate: transaction)) ].
	(amounts x roundTo: (1/100)) < 0 ifTrue: [ self addNoticeFor: account inSchedule: b shortBy: amounts x negated].
	(amounts y roundTo: (1/100)) < 0 ifTrue: [ self addNoticeFor: pool inSchedule: b shortBy: amounts y negated].! !

!ScAccountsSufficientForBills methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 17:57'!
updateNotices
	notices := OrderedCollection new.
	((profile schedules select: [ :s | s isExpense ])
		sorted: [ :s1 :s2 | s1 nextDate <= s2 nextDate ])
		do: [ :b | self testBill: b]! !

!ScAnalyser methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 18:54'!
notices
	^#()! !

!ScObject class methodsFor: 'instance creation' stamp: 'StuartHerring 12/11/2011 14:19'!
profile: aProfile
	^self basicNew
		initializeWithProfile: aProfile;
		yourself.! !

!ScObject methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:33'!
announce: anAnnouncement
	self modificationAnnouncer 
		ifNotNil: [ :a | a announce: anAnnouncement]! !

!ScObject methodsFor: 'initialization' stamp: 'StuartHerring 6/11/2012 19:43'!
initializeWithProfile: aProfile
	self initialize.
	profile := aProfile.
	announcer := nil.! !

!ScObject methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 19:44'!
modificationAnnouncer
	^announcer! !

!ScObject methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/17/2012 19:07'!
modificationAnnouncer: anAnnouncer
	announcer = anAnnouncer ifTrue: [ ^self ].
	
	announcer ifNotNil: [ :a | a unsubscribe: self].
	announcer := anAnnouncer.
	announcer ifNotNil: [ :a | self registerSubscriptionsOn: a ]
	! !

!ScObject methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:28'!
registerSubscriptionsOn: anAnnouncer! !

!ScObject methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 21:48'!
set: aSymbol to: aValue announceAs: anScItemModifiedAnnouncementClass
	| oldValue |
	oldValue := self perform: aSymbol.
	self set: aSymbol to: aValue.
	aValue = oldValue 
		ifFalse: [ self announce: (anScItemModifiedAnnouncementClass item: self field: aSymbol value: aValue oldValue: oldValue) ]! !

ScObject subclass: #ScSchedule
	instanceVariableNames: 'start interval count transaction name coveredBy manualTransactions timetable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScSchedule class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:45'!
intervalTypes
	^#(day week month year)! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 17:55'!
account
	^transaction account! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:02'!
account: anAccount
	transaction account: anAccount! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/10/2012 23:53'!
accumulate: anAmount inAccount: anAccount using: aDictionary
	| t |
	anAccount = transaction account
		ifTrue: [ ^ self ].
	t := aDictionary
		at: anAccount
		ifAbsentPut: [ 
			self newTransaction
				comment: 'Automatic transfer for ' , (self name ifNil: [ '<unnamed>' ]);
				sourceAccount: transaction account;
				account: anAccount;
				amount: 0 ].
	t amount: t amount + anAmount! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/10/2012 23:53'!
accumulate: anAmount inPool: aPool using: aDictionary
	| t |
	aPool = transaction pool
		ifTrue: [ ^ self ].
	t := aDictionary
		at: aPool
		ifAbsentPut: [ 
			self newTransaction
				comment: 'Automatic allocation for ' , (self name ifNil: [ '<unnamed>' ]);
				sourcePool: transaction pool;
				pool: aPool;
				amount: 0 ].
	t amount: t amount + anAmount! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/18/2012 18:30'!
addTo: aDate
	^timetable addTo: aDate! !

!ScSchedule methodsFor: 'actions' stamp: 'StuartHerring 3/23/2012 17:34'!
addTransaction: aTransaction
	manualTransactions add: aTransaction! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 17:55'!
amount
	^transaction amount! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:01'!
amount: aNumber
	transaction amount: aNumber! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/20/2012 22:41'!
automaticTransactions
	| poolTransactions accountTransactions |
	poolTransactions := Dictionary new.
	accountTransactions := Dictionary new.
	manualTransactions do: [ :t | 
		self accumulate: t amount inPool: t pool using: poolTransactions.
		self accumulate: t amount inAccount: t account using: accountTransactions].
	self billsCovered do: [ :b |  | amount |
		amount := b calculatedAmountOver: timetable.
		self accumulate: amount inPool:  b sourcePool using: poolTransactions.
		self accumulate: amount inAccount: b sourceAccount using: accountTransactions].
	
	^{poolTransactions. accountTransactions} gather: [ :t | t values collect: [ :tr | tr asTransaction ]]! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/8/2012 17:12'!
billsCovered
	"Answers the bills covered by this schedule if it's an income"
	^self isIncome ifFalse: [#()]
		ifTrue: [ profile schedules select: [ :p | p coveredBy = self ]]! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/20/2012 22:43'!
calculatedAmountOver: aTimetable
	^(self amount * (timetable ratioFor: aTimetable)) roundTo: (1/100)! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:22'!
count
	<required>
	<integer>
	^timetable count! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:39'!
count: aNumber
	| oldValue |
	oldValue := timetable count.
	self mbValidateField: #count with: aNumber.
	timetable count: aNumber.
	oldValue = aNumber ifFalse: [ self announce: (ScScheduleModified item: self field: #count value: aNumber oldValue: oldValue)]! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/17/2012 13:37'!
coveredBy
	<valueSelector: #validOwners>
	^coveredBy! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:41'!
coveredBy: aSchedule
	self 
		set: #coveredBy
		to: aSchedule
		announceAs: ScScheduleModified! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 3/5/2012 19:56'!
datesBetween: rangeStart and: rangeEnd
	| dates currentDate |
	dates := OrderedCollection new.
	currentDate := self nextAfter: rangeStart.
	[ currentDate > rangeEnd ] whileFalse: [
		dates add: currentDate.
		currentDate := self addTo: currentDate].
	^dates! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 7/29/2012 12:58'!
futureTransactionsBetween: rangeStart and: rangeEnd
	^((self datesBetween: rangeStart and: rangeEnd)
		select: [ :d | d > rangeStart])
			inject: OrderedCollection new 
			into: [ :result :d | result 
					add: (transaction copy date: d);
					addAll: (self automaticTransactions collect: [ :t | t date: d]) ;
					yourself]! !

!ScSchedule methodsFor: 'initialization' stamp: 'StuartHerring 7/29/2012 12:52'!
initializeWithProfile: aProfile
	<mbFields: #(name start interval count transaction coveredBy)>
	super initializeWithProfile: aProfile.
	transaction := ScTransaction profile: profile.
	manualTransactions := OrderedCollection new.
	timetable := ScMonthly every: 1.
	start := Date today.
	count := 1.! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:20'!
interval
	<valueSelector: #intervalTypes>
	<required>
	^timetable symbol! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:41'!
interval: aSymbol
	| oldValue |
	oldValue := timetable symbol.
	oldValue = aSymbol ifTrue: [ ^self ].

	self mbValidateField: #interval with: aSymbol.
	timetable := ScTimetable period: aSymbol count: timetable count.
	self announce: (ScScheduleModified item: self field: #interval value: aSymbol oldValue: oldValue)! !

!ScSchedule methodsFor: 'constraints' stamp: 'StuartHerring 6/11/2012 09:45'!
intervalTypes
	^self class intervalTypes! !

!ScSchedule methodsFor: 'testing' stamp: 'StuartHerring 3/17/2012 13:42'!
isAllocation
	^transaction isAllocation! !

!ScSchedule methodsFor: 'testing' stamp: 'StuartHerring 3/17/2012 13:42'!
isExpense
	^transaction isExpense! !

!ScSchedule methodsFor: 'testing' stamp: 'StuartHerring 3/17/2012 13:42'!
isIncome
	^transaction isIncome! !

!ScSchedule methodsFor: 'testing' stamp: 'StuartHerring 3/17/2012 13:42'!
isTransfer
	^transaction isTransfer! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/23/2012 17:33'!
manualTransactions
	^manualTransactions! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/17/2012 13:59'!
name
	<required>
	^name! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/4/2012 16:57'!
name: aString
	self set: #name to: aString! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/11/2012 00:24'!
newTransaction
	^ScFastTransaction  profile: profile! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 6/18/2012 18:30'!
nextAfter: aDate
	| newDate |
	newDate := start.
	[ newDate > aDate ] whileFalse: [ newDate := self addTo: newDate ].
	^newDate
	! !

!ScSchedule methodsFor: 'private' stamp: 'StuartHerring 3/5/2012 19:48'!
nextDate
	^self nextAfter: Date current.! !

!ScSchedule methodsFor: 'testing' stamp: 'StuartHerring 5/9/2012 21:20'!
ownsNotice: aNotice
	^aNotice schedule = self! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 7/29/2012 12:39'!
pendingTransactions
	^(profile pendingTransactions 
		select: [ :p | p schedule = self ])
		collect: [ :p | p transaction ]
		! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 17:55'!
pool
	^transaction pool! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:03'!
pool: aPool
	transaction pool: aPool! !

!ScSchedule methodsFor: 'printing' stamp: 'StuartHerring 3/18/2012 09:51'!
printOn: aStream
	super printOn: aStream.
	aStream 
		nextPut: $(;
		nextPutAll: (name ifNil: ['<unnamed>']);
		nextPut: $).! !

!ScSchedule methodsFor: 'actions' stamp: 'StuartHerring 3/23/2012 17:34'!
removeTransaction: aTransaction
	manualTransactions remove: aTransaction! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 17:55'!
sourceAccount
	^transaction sourceAccount! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:02'!
sourceAccount: anAccount
	transaction sourceAccount: anAccount! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 17:55'!
sourcePool
	^transaction sourcePool! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:03'!
sourcePool: aPool
	transaction sourcePool: aPool! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/4/2012 10:13'!
start
	<date>
	<required>
	^start! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 3/4/2012 10:14'!
start: aDate
	self set: #start to: aDate! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/20/2012 23:12'!
timetable
	^timetable! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 18:08'!
transaction
	<required>
	<embed>
	^transaction! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:37'!
transaction: aTransactionTemplate
	self 
		set: #transaction 
		to: aTransactionTemplate 
		announceAs: ScScheduleModified ! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 7/29/2012 12:58'!
transactionsBetween: rangeStart and: rangeEnd
	^(OrderedCollection withAll:  self pendingTransactions)
		addAll: (self futureTransactionsBetween: rangeStart and: rangeEnd) ;
		yourself! !

!ScSchedule methodsFor: 'accessing' stamp: 'StuartHerring 6/20/2012 22:42'!
unallocated
	^transaction amount - 
				(self billsCovered inject: 0 into: [ :total :bill | total + (bill calculatedAmountOver: timetable ) ]) -
				(self manualTransactions inject: 0 into: [ :total :t | total + (t amount)])! !

!ScSchedule methodsFor: 'constraints' stamp: 'StuartHerring 6/18/2012 17:48'!
validOwners
	^self isExpense 
		ifFalse: [ #() ]
		ifTrue: [ profile schedules select: [ :s | s isIncome ]]! !

ScObject subclass: #ScTransaction
	instanceVariableNames: 'comment amount date sourcePool sourceAccount pool account'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

ScTransaction subclass: #ScExpense
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScExpense methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 13:00'!
account
	<readOnly>
	^nil! !

!ScExpense methodsFor: 'testing' stamp: 'StuartHerring 6/9/2012 15:34'!
initialize 
	<mbFields: #(amount sourcePool sourceAccount comment date)>
	super initialize.
! !

!ScExpense methodsFor: 'testing' stamp: 'StuartHerring 2/15/2012 07:17'!
isExpense
	^true! !

!ScExpense methodsFor: 'testing' stamp: 'StuartHerring 2/15/2012 07:17'!
isIncome
	^false! !

!ScExpense methodsFor: 'testing' stamp: 'StuartHerring 2/25/2012 13:59'!
isTransfer
	^false! !

!ScExpense methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 13:00'!
pool
	<readOnly>
	^nil! !

!ScExpense methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 15:19'!
printOn: aStream
	aStream nextPutAll:  'Spent ';
		nextPutAll: '$'; 
		nextPutAll: amount printAsCurrency;
		nextPutAll: ' on ';
		nextPutAll: (sourcePool ifNil: [ 'Unallocated' ] ifNotNil: [ :p | p name ]);
		nextPutAll: ' from ';
		nextPutAll: sourceAccount name. ! !

ScTransaction subclass: #ScFastTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScFastTransaction commentStamp: '<historical>' prior: 0!
Automatic allocation for Pay Day!

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:01'!
account: anAccount
	account := anAccount! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:02'!
amount: aNumber
	amount := aNumber! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:01'!
comment: aString
	comment := aString! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:02'!
date: aDate
	date := aDate! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:00'!
pool: aPool
	pool := aPool! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:01'!
sourceAccount: anAccount
	sourceAccount := anAccount! !

!ScFastTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 00:00'!
sourcePool: aPool
	sourcePool := aPool! !

ScTransaction subclass: #ScIncome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScIncome methodsFor: 'testing' stamp: 'StuartHerring 6/9/2012 15:35'!
initialize 
	<mbFields: #(amount pool account comment date)>	
	super initialize.
! !

!ScIncome methodsFor: 'testing' stamp: 'StuartHerring 2/15/2012 07:17'!
isExpense
	^false! !

!ScIncome methodsFor: 'testing' stamp: 'StuartHerring 2/15/2012 07:15'!
isIncome
	^true! !

!ScIncome methodsFor: 'testing' stamp: 'StuartHerring 2/25/2012 13:59'!
isTransfer
	^false! !

!ScIncome methodsFor: 'accessing' stamp: 'StuartHerring 3/11/2012 11:12'!
printOn: aStream
	aStream nextPutAll: 'Received $';
		nextPutAll: amount printAsCurrency;
		nextPutAll: ' for ';
		nextPutAll: (pool ifNil: [ 'Unallocated' ] ifNotNil: [ :p | p name ]);
		nextPutAll: ' in ';
		nextPutAll: account name. ! !

!ScIncome methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 13:00'!
sourceAccount
	<readOnly>
	^nil! !

!ScIncome methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 13:00'!
sourcePool
	<readOnly>
	^nil! !

!ScTransaction methodsFor: 'comparing' stamp: 'StuartHerring 4/14/2012 08:28'!
<= b
	^date = b date 
		ifTrue: [ self isIncome ]
		ifFalse: [date < b date]! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2012 18:20'!
account
	<valueSelector: #accountList>
	<required>
	^account! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:43'!
account: anAccount
	self 
		set: #account 
		to: anAccount
		announceAs: ScTransactionModified! !

!ScTransaction methodsFor: 'constraints' stamp: 'StuartHerring 1/29/2012 10:43'!
accountList
	^profile accounts! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 2/6/2012 21:54'!
amount
	<required>
	<currency: $$>
	^amount! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:37'!
amount: anAmount
	self 
		set: #amount 
		to: anAmount
		announceAs: ScTransactionBalanceModified! !

!ScTransaction methodsFor: 'converting' stamp: 'StuartHerring 6/4/2012 20:49'!
asExpense
	^(ScExpense profile: profile)
		amount: amount;
		sourcePool: sourcePool;
		sourceAccount: sourceAccount;
		comment: comment;
		date: date;
		yourself! !

!ScTransaction methodsFor: 'converting' stamp: 'StuartHerring 6/4/2012 20:49'!
asIncome
	^(ScIncome profile: profile)
		amount: amount;
		pool: pool;
		account: account;
		comment: comment;
		date: date;
		yourself! !

!ScTransaction methodsFor: 'converting' stamp: 'StuartHerring 6/11/2012 00:14'!
asTransaction
	^ScTransaction basicNew copyFrom: self.! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 12/11/2011 15:06'!
comment
	^comment! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2012 17:05'!
comment: aComment
	self set: #comment to: aComment! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 2/6/2012 21:54'!
date
	<date>
	^date! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2012 17:05'!
date: aDateAndTime
	self set: #date to: aDateAndTime! !

!ScTransaction methodsFor: 'initialization' stamp: 'StuartHerring 6/3/2012 21:07'!
initialize
	<mbFields: #(amount sourcePool sourceAccount pool account comment date)>
	super initialize.
	date := Date today.
	amount := 0.! !

!ScTransaction methodsFor: 'testing' stamp: 'StuartHerring 5/21/2012 18:49'!
isAllocation
	^ self isIncome not & self isExpense not & (pool ~= sourcePool)! !

!ScTransaction methodsFor: 'testing' stamp: 'StuartHerring 3/18/2012 14:56'!
isExpense
	^account isNil & sourceAccount notNil! !

!ScTransaction methodsFor: 'testing' stamp: 'StuartHerring 3/18/2012 14:55'!
isIncome
	^sourceAccount isNil & account notNil! !

!ScTransaction methodsFor: 'testing' stamp: 'StuartHerring 5/21/2012 18:49'!
isTransfer
	^ self isIncome not & self isExpense not & (account ~= sourceAccount)! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2012 18:31'!
pool
	<valueSelector: #poolList>
	^pool! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:44'!
pool: aPool
	self set: #pool to: aPool announceAs: ScTransactionModified! !

!ScTransaction methodsFor: 'constraints' stamp: 'StuartHerring 1/30/2012 18:31'!
poolList
	^profile pools! !

!ScTransaction methodsFor: 'printing' stamp: 'StuartHerring 5/21/2012 18:49'!
printOn: aStream
	| allocate transfer |
	transfer := sourceAccount ~= account.
	allocate := sourcePool ~= pool.
	allocate
		ifTrue: [ 
			aStream nextPutAll: 'Allocated '.
			transfer
				ifTrue: [ aStream nextPutAll: ' and ' ] ].
	transfer
		ifTrue: [ aStream nextPutAll: 'Transfered ' ].
	aStream
		nextPutAll: '$';
		nextPutAll: amount printAsCurrency.
	allocate & transfer not | (transfer & allocate not)
		ifFalse: [ 
			aStream nextPutAll: ' from '.
			allocate
				ifTrue: [ 
					aStream nextPutAll: (sourcePool ifNil: [ 'Unallocated' ] ifNotNil: [ :p | p name ]).
					transfer
						ifTrue: [ aStream nextPutAll: ' in ' ] ].
			transfer
				ifTrue: [ aStream nextPutAll: sourceAccount name ] ].
	aStream nextPutAll: ' to '.
	allocate
		ifTrue: [ 
			aStream nextPutAll: (pool ifNil: [ 'Unallocated' ] ifNotNil: [ :p | p name ]).
			transfer
				ifTrue: [ aStream nextPutAll: ' in ' ] ].
	transfer
		ifTrue: [ aStream nextPutAll: account name ]! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 07:13'!
sourceAccount
	<valueSelector: #accountList>
	<required>
	^sourceAccount! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:43'!
sourceAccount: anAccount
	self set: #sourceAccount to: anAccount announceAs: ScTransactionModified! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 2/15/2012 07:13'!
sourcePool
	<valueSelector: #poolList>
	^sourcePool! !

!ScTransaction methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:43'!
sourcePool: aPool
	self set: #sourcePool to: aPool announceAs: ScTransactionModified! !

ScObject subclass: #ScTransactionHolder
	instanceVariableNames: 'name balance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

ScTransactionHolder subclass: #ScAccount
	instanceVariableNames: 'openingBalance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScAccount methodsFor: 'calculating' stamp: 'StuartHerring 3/5/2012 20:38'!
accumulate: aTransaction
	^((aTransaction sourceAccount = aTransaction account) or: [ (self ownsTransaction: aTransaction) not]) ifTrue: [0]
		ifFalse: [
			(aTransaction account = self) 
				ifTrue: [aTransaction amount]
				ifFalse: [aTransaction amount negated]]! !

!ScAccount methodsFor: 'initialization' stamp: 'StuartHerring 3/3/2012 21:54'!
initialize
	<mbFields: #(name openingBalance)>
	super initialize.
	openingBalance := 0.! !

!ScAccount methodsFor: 'testing' stamp: 'StuartHerring 6/13/2012 21:02'!
isAccount
	^true! !

!ScAccount methodsFor: 'testing' stamp: 'StuartHerring 6/20/2012 21:25'!
isIncoming: aTransactionOrSchedule
	^aTransactionOrSchedule account = self and: [(aTransactionOrSchedule sourceAccount = self) not]! !

!ScAccount methodsFor: 'testing' stamp: 'StuartHerring 6/20/2012 21:25'!
isOutgoing: aTransactionOrSchedule
	^aTransactionOrSchedule sourceAccount = self and: [(aTransactionOrSchedule account = self) not]! !

!ScAccount methodsFor: 'accessing' stamp: 'StuartHerring 2/12/2012 19:56'!
openingBalance
	<label: 'Opening Balance'>
	<currency: $$>
	<required>
	^openingBalance! !

!ScAccount methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:34'!
openingBalance: anAmount
	| old |
	anAmount = openingBalance ifTrue: [ ^self ].
	
	old := openingBalance.
	openingBalance := anAmount.
	self updateBalance.
	self announce: (ScOpeningBalanceChanged item: self value: openingBalance oldValue: old)! !

!ScAccount methodsFor: 'testing' stamp: 'StuartHerring 5/9/2012 21:20'!
ownsNotice: aNotice
	^aNotice account = self! !

!ScAccount methodsFor: 'testing' stamp: 'StuartHerring 6/23/2012 14:34'!
transactionDescriptionFor: aTransaction
	^aTransaction isExpense 
		ifTrue: [ 'Expense' ]
		ifFalse: [ aTransaction isIncome 
				ifTrue: ['Income']
				ifFalse: [(self isIncoming: aTransaction) 
					ifTrue: ['Transfer from ', aTransaction sourceAccount name]
					ifFalse: ['Transfer to ', aTransaction account name ]]]! !

ScAccount subclass: #ScCreditAccount
	instanceVariableNames: 'creditLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScCreditAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:13'!
initialize
	<mbFields: #(name openingBalance creditLimit)>
	super initialize.
	creditLimit := 0.! !

ScTransactionHolder subclass: #ScAllTransactions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/16/2012 12:08'!
accumulate: aTransaction
	^aTransaction isExpense ifTrue: [aTransaction amount negated] 
		ifFalse: [aTransaction isIncome ifTrue: [aTransaction amount] ifFalse: [0]]! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/24/2012 16:39'!
isIncoming: aTransaction
	^aTransaction isIncome.! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/24/2012 16:39'!
isOutgoing: aTransaction
	^aTransaction isExpense! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/24/2012 16:41'!
name
	^'All Transactions'! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/21/2012 20:16'!
openingBalance
	^ profile accounts inject: 0 into: [ :sum :a | sum + a openingBalance ]! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/15/2012 21:14'!
profile: aProfile
	profile := aProfile! !

!ScAllTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/15/2012 21:12'!
transactions
	^profile transactions! !

ScTransactionHolder subclass: #ScPool
	instanceVariableNames: 'defaultAccount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScPool methodsFor: 'constraints' stamp: 'StuartHerring 2/7/2012 21:47'!
accountList
	^profile accounts! !

!ScPool methodsFor: 'calculating' stamp: 'StuartHerring 3/5/2012 20:38'!
accumulate: aTransaction
	^((aTransaction sourcePool = aTransaction pool) or: [ (self ownsTransaction: aTransaction) not]) ifTrue: [0]
		ifFalse: [
			(aTransaction pool = self) 
				ifTrue: [aTransaction amount]
				ifFalse: [aTransaction amount negated]]! !

!ScPool methodsFor: 'accessing' stamp: 'StuartHerring 2/7/2012 21:50'!
defaultAccount
	<required>
	<valueSelector: #accountList>
	^defaultAccount! !

!ScPool methodsFor: 'accessing' stamp: 'StuartHerring 2/7/2012 21:48'!
defaultAccount: anAccount
	self set: #defaultAccount to: anAccount! !

!ScPool methodsFor: 'initialization' stamp: 'StuartHerring 3/3/2012 21:55'!
initialize
	<mbFields: #(name defaultAccount)>
	super initialize.! !

!ScPool methodsFor: 'testing' stamp: 'StuartHerring 6/20/2012 21:33'!
isIncoming: aTransactionOrSchedule
	^aTransactionOrSchedule pool = self 
		and: [ (aTransactionOrSchedule sourcePool = self) not ]! !

!ScPool methodsFor: 'testing' stamp: 'StuartHerring 6/20/2012 21:33'!
isOutgoing: aTransactionOrSchedule
	^aTransactionOrSchedule sourcePool = self 
		and: [ (aTransactionOrSchedule pool = self) not ]! !

!ScPool methodsFor: 'initialization' stamp: 'StuartHerring 6/13/2012 21:02'!
isPool
	^true! !

!ScPool methodsFor: 'testing' stamp: 'StuartHerring 5/9/2012 21:20'!
ownsNotice: aNotice
	^aNotice pool = self! !

!ScPool methodsFor: 'testing' stamp: 'StuartHerring 6/23/2012 14:33'!
transactionDescriptionFor: aTransaction
	^aTransaction isExpense 
		ifTrue: [ 'Expense' ]
		ifFalse: [ aTransaction isIncome 
				ifTrue: ['Income']
				ifFalse: [(self isIncoming: aTransaction) 
					ifTrue: [ aTransaction sourcePool ifNil: ['Allocated'] ifNotNil: [ :p | 'Reallocated from ', p name ] ]
					ifFalse: [aTransaction pool ifNil: ['Unallocated'] ifNotNil: [ :p | 'Reallocated to ', p name ]]]]! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 2/15/2012 13:20'!
accumulate: aTransaction
	"Return the amount to add or subtract from the balance for this transaction"
	self subclassResponsibility! !

!ScTransactionHolder methodsFor: 'printing' stamp: 'StuartHerring 2/7/2012 21:45'!
asString
	^name! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 23:26'!
averageIncomingPer: aTimePeriod
	| transactions duration sum |
	transactions := self transactions sorted.
	transactions ifEmpty: [^0].
	duration := (transactions last date - transactions first date) + 1 day.
	sum := (transactions sum: [ :t | (self isIncoming: t) ifTrue: [t amount ] ifFalse: [0] ]).
	^(duration days < aTimePeriod inDays) ifTrue: [ sum ]
		ifFalse: [ (sum / duration days) * aTimePeriod inDays ].
! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 23:26'!
averageOutgoingPer: aTimePeriod
	| transactions duration sum |
	transactions := self transactions sorted.
	transactions ifEmpty: [^0].
	duration := (transactions last date - transactions first date) + 1 day.
	sum := (transactions sum: [ :t | (self isOutgoing: t) ifTrue: [t amount ] ifFalse: [0] ]).
	^(duration days < aTimePeriod inDays) ifTrue: [ sum ]
		ifFalse: [ (sum / duration days) * aTimePeriod inDays ].
! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/7/2012 22:03'!
averageSpendPer: aTimePeriod
	| transactions duration sum |
	transactions := self transactions sorted.
	transactions ifEmpty: [^0].
	duration := (transactions last date - transactions first date) + 1 day.
	sum := (transactions sum: [ :t | t isExpense ifTrue: [t amount ] ifFalse: [0] ]).
	^(duration days < aTimePeriod inDays) ifTrue: [ sum ]
		ifFalse: [ (sum / duration days) * aTimePeriod inDays ].
! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 23:44'!
averageTransfersPer: aTimePeriod
	| transactions duration sum |
	transactions := self transactions sorted.
	transactions ifEmpty: [^0].
	duration := (transactions last date - transactions first date) + 1 day.
	sum := (transactions sum: [ :t | t isTransfer ifTrue: [self accumulate: t ] ifFalse: [0] ]).
	^(duration days < aTimePeriod inDays) ifTrue: [ sum ]
		ifFalse: [ (sum / duration days) * aTimePeriod inDays ].
! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 19:21'!
balance
	<ignore>
	^balance
! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/27/2012 20:02'!
balanceAt: aDate
	"Answers the opening balance for a specified date"
	^self openingBalance + (self sumOf: (self transactions select: [ :t | t date < aDate]))! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/9/2012 14:19'!
expectedBalanceBy: aDate
	^self balance + (self sumOf: (self expectedTransactionsBy: aDate))! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/9/2012 14:21'!
expectedTransactionsBy: aDate
	^(profile expectedTransactionsBy: aDate)
		select: [ :t | self ownsTransaction: t]! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 21:20'!
incomingSchedules
	^profile schedules
		select: [ :s | self isIncoming: s ]! !

!ScTransactionHolder methodsFor: 'initialization' stamp: 'StuartHerring 6/18/2012 19:54'!
initialize
	<mbFields: #(name)>
	super initialize.
	name := ''.
	balance := 0.! !

!ScTransactionHolder methodsFor: 'testing' stamp: 'StuartHerring 6/13/2012 21:02'!
isAccount
	^false! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 21:21'!
isIncoming: aTransactionOrSchedule
	^false! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 21:21'!
isOutgoing: aTransactionOrSchedule
	^false! !

!ScTransactionHolder methodsFor: 'testing' stamp: 'StuartHerring 6/13/2012 21:02'!
isPool
	^false! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 2/7/2012 21:45'!
name
	<label: 'Name'>
	<required>
	^name! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 2/7/2012 21:46'!
name: aName
	name := aName! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 2/12/2012 19:57'!
openingBalance
	<ignore>
	^0! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 21:20'!
outgoingSchedules
	^profile schedules
		select: [ :s | self isOutgoing: s]! !

!ScTransactionHolder methodsFor: 'testing' stamp: 'StuartHerring 5/9/2012 21:20'!
ownsNotice: aNotice
	^false! !

!ScTransactionHolder methodsFor: 'testing' stamp: 'StuartHerring 6/20/2012 21:35'!
ownsTransaction: t
	^(self isOutgoing: t) or: [ self isIncoming: t ]! !

!ScTransactionHolder methodsFor: 'printing' stamp: 'StuartHerring 3/17/2012 15:08'!
printOn: aStream
	super printOn: aStream.
	aStream 
		nextPut: $(;
		nextPutAll: name;
		nextPut: $)! !

!ScTransactionHolder methodsFor: 'initialization' stamp: 'StuartHerring 6/18/2012 21:50'!
registerSubscriptionsOn: anAnnouncer
	anAnnouncer
		on: ScTransactionAdded do: [ : a | self transactionAdded: a transaction];
		on: ScTransactionRemoved do: [ :a |self transactionRemoved: a transaction];
		on: ScTransactionBalanceModified do: [ :a | self transactionModified: a ];
		on: ScTransactionModified do: [ :a | self transactionModified: a ]! !

!ScTransactionHolder methodsFor: 'calculating' stamp: 'StuartHerring 6/20/2012 22:20'!
relatedSchedules
	^profile schedules
		select: [ :s | self ownsTransaction: s]! !

!ScTransactionHolder methodsFor: 'private' stamp: 'StuartHerring 6/9/2012 14:17'!
sumOf: transactions
	^transactions inject: 0 into: [ :total :this | total + (self accumulate: this) ]! !

!ScTransactionHolder methodsFor: 'events' stamp: 'StuartHerring 6/25/2012 21:47'!
transactionAdded: aTransaction
	(self ownsTransaction: aTransaction)
		ifTrue: [ self updateBalance ]! !

!ScTransactionHolder methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/23/2012 10:20'!
transactionDescriptionFor: aTransaction
	^aTransaction printString! !

!ScTransactionHolder methodsFor: 'events' stamp: 'StuartHerring 6/18/2012 22:01'!
transactionModified: anAnnouncement
	(#(sourcePool sourceAccount pool account amount) 
		includes: anAnnouncement field) ifTrue: [ self updateBalance ].! !

!ScTransactionHolder methodsFor: 'events' stamp: 'StuartHerring 6/18/2012 19:25'!
transactionRemoved: aTransaction
	(self ownsTransaction: aTransaction)
		ifTrue: [ self updateBalance ]! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 2/14/2012 16:26'!
transactions
	<ignore>
		^profile transactions select: [ :t | self ownsTransaction: t ]! !

!ScTransactionHolder methodsFor: 'accessing' stamp: 'StuartHerring 5/18/2012 23:32'!
transactionsBetween: startDate and: endDate
		^(profile transactionsBetween: startDate and: endDate) select: [ :t | self ownsTransaction: t ]! !

!ScTransactionHolder methodsFor: 'private' stamp: 'StuartHerring 6/25/2012 21:47'!
updateBalance
	| oldBalance |
	oldBalance := balance.
	balance := self openingBalance + (self sumOf: self transactions).
	balance = oldBalance ifFalse: 
		[ self announce: (ScBalanceChanged item: self field: #balance value: balance oldValue: oldBalance) ]! !

ScTransactionHolder subclass: #ScUnallocated
	instanceVariableNames: 'openingBalance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScUnallocated class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 19:56'!
new
	self shouldNotImplement ! !

!ScUnallocated methodsFor: 'calculating' stamp: 'StuartHerring 3/17/2012 17:16'!
accumulate: aTransaction
	^(self ownsTransaction: aTransaction) ifFalse: [0]
		ifTrue: [ aTransaction sourcePool isNil 
			ifTrue: [ aTransaction isIncome ifTrue: [aTransaction amount] ifFalse: [aTransaction amount negated ]] 
			ifFalse: [aTransaction amount ] ]! !

!ScUnallocated methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 19:58'!
initializeWithProfile: aProfile
	super initializeWithProfile: aProfile.
	self updateOpeningBalance.
	self updateBalance! !

!ScUnallocated methodsFor: 'accessing' stamp: 'StuartHerring 6/20/2012 22:07'!
isIncoming: aTransactionOrSchedule
	^aTransactionOrSchedule isExpense not
		and: [ aTransactionOrSchedule pool isNil 
			and: [ aTransactionOrSchedule sourcePool notNil or: [ aTransactionOrSchedule isIncome ]]]! !

!ScUnallocated methodsFor: 'accessing' stamp: 'StuartHerring 6/20/2012 22:06'!
isOutgoing: aTransactionOrSchedule
	^aTransactionOrSchedule isIncome not
		and: [ aTransactionOrSchedule sourcePool isNil 
			and: [ aTransactionOrSchedule pool notNil or: [ aTransactionOrSchedule isExpense ]]]! !

!ScUnallocated methodsFor: 'testing' stamp: 'StuartHerring 6/13/2012 21:02'!
isPool
	^true! !

!ScUnallocated methodsFor: 'accessing' stamp: 'StuartHerring 3/17/2012 15:43'!
name
	^'Unallocated'! !

!ScUnallocated methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 19:58'!
openingBalance
	^openingBalance! !

!ScUnallocated methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:29'!
registerSubscriptionsOn: anAnnouncer
	super registerSubscriptionsOn: anAnnouncer.
	anAnnouncer 
		on: ScAccountAdded, ScAccountRemoved, ScOpeningBalanceChanged 
		send: #updateOpeningBalance to: self.! !

!ScUnallocated methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 21:23'!
updateOpeningBalance
	| old |
	old := openingBalance.
	openingBalance := profile accounts inject: 0 into: [ :total :account | total + account openingBalance].
	old = openingBalance ifFalse: [ self updateBalance ].! !

Notification subclass: #ScInstrumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Development'!

Announcement subclass: #ScItemAdded
	instanceVariableNames: 'item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

ScItemAdded subclass: #ScAccountAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScAccountAdded class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:03'!
account: anAccount
	^self item: anAccount! !

!ScAccountAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:03'!
account
	^self item! !

!ScItemAdded class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:57'!
item: anObject
	^self basicNew
		initializeWithItem: anObject;
		yourself.! !

!ScItemAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:58'!
initializeWithItem: anItem
	self initialize.
	item := anItem! !

!ScItemAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:58'!
item
	^item! !

ScItemAdded subclass: #ScPoolAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScPoolAdded class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:04'!
pool: aPool
	^self item: aPool! !

!ScPoolAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:04'!
pool
	^self item! !

ScItemAdded subclass: #ScScheduleAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScScheduleAdded class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:03'!
schedule: aSchedule
	^self item: aSchedule! !

!ScScheduleAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:04'!
schedule
	^self item! !

ScItemAdded subclass: #ScTransactionAdded
	instanceVariableNames: 'transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScTransactionAdded class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:02'!
transaction: aTransaction
	^self item: aTransaction! !

!ScTransactionAdded methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:01'!
transaction
	^self item! !

Announcement subclass: #ScItemModified
	instanceVariableNames: 'item field oldValue value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

ScItemModified subclass: #ScBalanceChanged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScBalanceChanged methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:14'!
account
	^item! !

!ScItemModified class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:09'!
item: anObject 
	^self item: anObject field: nil value: nil oldValue: nil! !

!ScItemModified class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:09'!
item: anObject field: aSymbol 
	^self item: anObject field: aSymbol value: nil oldValue: nil! !

!ScItemModified class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 21:28'!
item: anObject field: aSymbol oldValue: oldValueObject
	^self item: anObject field: aSymbol value: (anObject perform: aSymbol) oldValue: oldValueObject! !

!ScItemModified class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:09'!
item: anObject field: aSymbol value: valueObject
	^self item: anObject field: aSymbol value: valueObject oldValue: nil! !

!ScItemModified class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:07'!
item: anObject field: aSymbol value: valueObject oldValue: oldValueObject
	^self basicNew
		initializeWith: anObject field: aSymbol value: valueObject oldValue: oldValueObject;
		yourself.! !

!ScItemModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:08'!
field
	^field! !

!ScItemModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:08'!
initializeWith: anObject field:  aSymbol value:  valueObject oldValue: oldValueObject
	self initialize.
	item := anObject.
	field := aSymbol.
	value := valueObject.
	oldValue := oldValueObject.! !

!ScItemModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:08'!
item
	^item! !

!ScItemModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:08'!
oldValue
	^oldValue! !

!ScItemModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:08'!
value
	^value! !

ScItemModified subclass: #ScOpeningBalanceChanged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScOpeningBalanceChanged class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 21:34'!
item: anObject value: openingBalance oldValue: oldOpeningBalance
	^self item: anObject field: #openingBalance value: openingBalance oldValue: oldOpeningBalance! !

!ScOpeningBalanceChanged methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:16'!
account
	^item! !

ScItemModified subclass: #ScScheduleModified
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScScheduleModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:13'!
schedule 
	^item! !

ScItemModified subclass: #ScTransactionBalanceModified
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScTransactionBalanceModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 20:11'!
transaction
	^item! !

ScItemModified subclass: #ScTransactionModified
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScTransactionModified methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 21:26'!
transaction
	^item! !

Announcement subclass: #ScItemRemoved
	instanceVariableNames: 'item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

ScItemRemoved subclass: #ScAccountRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScAccountRemoved class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
account: anAccount
	^self item: anAccount! !

!ScAccountRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
account
	^self item! !

!ScItemRemoved class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:59'!
item: anObject
	^self basicNew
		initializeWithItem: anObject;
		yourself.! !

!ScItemRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:59'!
initializeWithItem: anItem
	self initialize.
	item := anItem! !

!ScItemRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:59'!
item
	^item! !

ScItemRemoved subclass: #ScPoolRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScPoolRemoved class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
pool: aPool
	^self item: aPool! !

!ScPoolRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
pool
	^self item! !

ScItemRemoved subclass: #ScScheduleRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScScheduleRemoved class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
schedule: aSchedule
	^self item: aSchedule! !

!ScScheduleRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:05'!
schedule
	^self item! !

ScItemRemoved subclass: #ScTransactionRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Announcements'!

!ScTransactionRemoved class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:02'!
transaction: aTransaction
	^self item: aTransaction! !

!ScTransactionRemoved methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 22:02'!
transaction
	^self item! !

MbCommand subclass: #ScAddManualTransaction
	instanceVariableNames: 'schedule transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScAddManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:08'!
do
	schedule addTransaction: transaction! !

!ScAddManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:26'!
name
	^'Add Manual Transaction'! !

!ScAddManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:07'!
schedule: aSchedule
	schedule := aSchedule! !

!ScAddManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:07'!
transaction: aTransaction
	transaction := aTransaction! !

!ScAddManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:34'!
undo
	schedule removeTransaction: transaction! !

MbCommand subclass: #ScModelCommand
	instanceVariableNames: 'profile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

ScModelCommand subclass: #ScAcceptTransactions
	instanceVariableNames: 'transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScAcceptTransactions class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 12:07'!
profile: aProfile transactions: aCollection
	^self new
		profile: aProfile;
		transactions: aCollection;
		yourself.! !

!ScAcceptTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/12/2012 19:14'!
do
	transactions do: [ :t | profile addTransaction: t transaction.
		profile pendingTransactions remove: t]! !

!ScAcceptTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/29/2012 21:09'!
name
	^'Accept Transactions'! !

!ScAcceptTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/4/2012 19:34'!
printOn: aStream
	aStream nextPutAll: 'Accepted pending transactions'! !

!ScAcceptTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/12/2012 19:13'!
transactions: aCollection
	transactions := aCollection copy! !

!ScAcceptTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 12:10'!
undo
	transactions do: [ :t | profile removeTransaction: t transaction.
		profile pendingTransactions add: t]! !

ScModelCommand subclass: #ScAddAccount
	instanceVariableNames: 'account'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScAddAccount class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:30'!
profile: aProfile account: anAccount
	^self new
		profile: aProfile;
		account: anAccount;
		yourself.! !

!ScAddAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:29'!
account: anAccount
	account := anAccount! !

!ScAddAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:30'!
do
	profile addAccount: account! !

!ScAddAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/29/2012 21:10'!
name
	^'Add Account'! !

!ScAddAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:33'!
printOn: aStream
	aStream nextPutAll: 'Added account named ';
		nextPutAll: account name.! !

!ScAddAccount methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:31'!
undo
	profile removeAccount: account! !

ScModelCommand subclass: #ScAddPool
	instanceVariableNames: 'pool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScAddPool class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:31'!
profile: aProfile pool: aPool
	^self new
		profile: aProfile;
		pool: aPool;
		yourself.! !

!ScAddPool methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:32'!
do
	profile addPool: pool! !

!ScAddPool methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:24'!
name
	^'Add Pool'! !

!ScAddPool methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:32'!
pool: aPool
	pool := aPool! !

!ScAddPool methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:33'!
printOn: aStream
	aStream nextPutAll: 'Added pool named ';
		nextPutAll: pool name.! !

!ScAddPool methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:32'!
undo
	profile removePool: pool! !

ScModelCommand subclass: #ScAddSchedule
	instanceVariableNames: 'schedule'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScAddSchedule class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/4/2012 17:15'!
profile: aProfile schedule: aSchedule
	^self new
		profile: aProfile;
		schedule: aSchedule;
		yourself.! !

!ScAddSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/4/2012 17:16'!
do
	profile addSchedule: schedule! !

!ScAddSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:24'!
name
	^'Add Schedule'! !

!ScAddSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/4/2012 19:33'!
printOn: aStream
	aStream nextPutAll: 'Added schedule named ';
		nextPutAll: schedule name.! !

!ScAddSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/4/2012 17:16'!
schedule: aSchedule
	schedule := aSchedule! !

!ScAddSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/4/2012 17:17'!
undo
	profile removeSchedule: schedule! !

ScModelCommand subclass: #ScDeletePendingTransactions
	instanceVariableNames: 'transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScDeletePendingTransactions class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/12/2012 19:32'!
profile: aProfile transactions: aCollection
	^self new
		profile: aProfile;
		transactions: aCollection;
		yourself.! !

!ScDeletePendingTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 7/29/2012 13:07'!
do
	profile removePendingTransactions: transactions! !

!ScDeletePendingTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:25'!
name
	^'Delete Pending Transactions'! !

!ScDeletePendingTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/12/2012 19:32'!
transactions: aCollection
	transactions := aCollection copy! !

!ScDeletePendingTransactions methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/12/2012 19:32'!
undo
	profile pendingTransactions addAll: transactions! !

ScModelCommand subclass: #ScDeleteSchedule
	instanceVariableNames: 'schedule'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScDeleteSchedule class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/31/2012 10:06'!
profile: aProfile schedule: aSchedule
	^self new
		profile: aProfile;
		schedule: aSchedule;
		yourself.! !

!ScDeleteSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/31/2012 10:07'!
do
	profile removeSchedule: schedule! !

!ScDeleteSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:25'!
name
	^'Delete Schedule'! !

!ScDeleteSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/31/2012 10:06'!
printOn: aStream
	aStream nextPutAll: 'Added schedule named ';
		nextPutAll: schedule name.! !

!ScDeleteSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/31/2012 10:06'!
schedule: aSchedule
	schedule := aSchedule! !

!ScDeleteSchedule methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/31/2012 10:07'!
undo
	profile addSchedule: schedule! !

ScModelCommand subclass: #ScDeleteTransaction
	instanceVariableNames: 'transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScDeleteTransaction class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 4/13/2012 07:53'!
profile: aProfile transaction: aTransaction
	^self new
		profile: aProfile; transaction: aTransaction;
		yourself! !

!ScDeleteTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 4/13/2012 07:54'!
do
	profile removeTransaction: transaction.! !

!ScDeleteTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:25'!
name
	^'Delete Transaction'! !

!ScDeleteTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 4/13/2012 07:56'!
printOn: aStream
	aStream nextPutAll: 'Deleted a transaction'! !

!ScDeleteTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 4/13/2012 07:53'!
transaction: aTransaction
	transaction := aTransaction.! !

!ScDeleteTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 4/13/2012 07:55'!
undo
	profile addTransaction: transaction.! !

!ScModelCommand methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:28'!
profile: aProfile
	profile := aProfile! !

ScModelCommand subclass: #ScRecordTransaction
	instanceVariableNames: 'transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScRecordTransaction class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:29'!
profile: aProfile transaction: aTransaction
	^self new
		profile: aProfile; transaction: aTransaction;
		yourself! !

!ScRecordTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/14/2012 16:08'!
do
	profile addTransaction: transaction.! !

!ScRecordTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:25'!
name
	^'Record Transaction'! !

!ScRecordTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/15/2012 18:55'!
printOn: aStream
	transaction ifNotNil: [ :t | t printOn: aStream]! !

!ScRecordTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/12/2012 20:28'!
transaction: aTransaction
	transaction := aTransaction.! !

!ScRecordTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 2/14/2012 16:09'!
undo
	profile removeTransaction: transaction.! !

MbCommand subclass: #ScRemoveManualTransaction
	instanceVariableNames: 'schedule transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Commands'!

!ScRemoveManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:34'!
do
	schedule removeTransaction: transaction! !

!ScRemoveManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/1/2012 22:26'!
name
	^'Remove Manual Transaction'! !

!ScRemoveManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:12'!
schedule: aSchedule
	schedule := aSchedule! !

!ScRemoveManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:12'!
transaction: aTransaction
	transaction := aTransaction! !

!ScRemoveManualTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/24/2012 18:35'!
undo
	schedule addTransaction: transaction! !

!Number methodsFor: '*smallchange-core' stamp: 'StuartHerring 12/11/2011 11:28'!
printAsCurrency
	| s|
	s := WriteStream with: ''.
	self printOn: s showingDecimalPlaces: 2.
	^s contents! !

Object subclass: #ScAnalyserNotice
	instanceVariableNames: 'pool account schedule message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScAnalyserNotice class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:45'!
severity
	^#notice! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:16'!
account
	^account! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:15'!
account: anAccount
	account := anAccount! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 21:19'!
appliesTo: anObject
	^anObject ownsNotice: self! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:17'!
initialize
	super initialize.
	pool := nil.
	account := nil.
	schedule := nil! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:44'!
isNotice
	^self severity = #notice! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:44'!
isUrgent
	^self severity = #urgent! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:44'!
isWarning
	^self severity = #warning! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:18'!
message
	^message! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:18'!
message: aMessage
	message := aMessage! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:16'!
pool
	^pool! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:12'!
pool: aPool
	pool := aPool! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:16'!
schedule
	^schedule! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/14/2012 19:15'!
schedule: aSchedule
	schedule := aSchedule.! !

!ScAnalyserNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:45'!
severity
	^self class severity! !

ScAnalyserNotice subclass: #ScInsufficientFundsNotice
	instanceVariableNames: 'amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScInsufficientFundsNotice class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:45'!
severity
	^#warning! !

!ScInsufficientFundsNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/18/2012 10:12'!
amount
	^amount! !

!ScInsufficientFundsNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/18/2012 10:12'!
amount: anAmount
	amount := anAmount.! !

!ScInsufficientFundsNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/18/2012 10:16'!
message
	| source sourceName | 
	sourceName := 'account'.
	source := account ifNil: [ sourceName := 'pool'. pool].
	^'Not enough money in {1} {2} to pay {3} on {4}. Short by: {5}' 
		format: { sourceName. source name. schedule name. schedule nextDate. amount printAsCurrency }! !

ScAnalyserNotice subclass: #ScNegativeBalanceNotice
	instanceVariableNames: 'amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core-Analysers'!

!ScNegativeBalanceNotice class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/13/2012 21:00'!
account: anAccount
	| symbol |
	symbol := anAccount isAccount 
		ifTrue: [ #account: ]
		ifFalse: [ #pool: ].
	^self new
		amount: anAccount balance;
		perform: symbol with: anAccount;
		yourself.
		! !

!ScNegativeBalanceNotice class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:45'!
severity
	^#urgent! !

!ScNegativeBalanceNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:02'!
amount
	^amount! !

!ScNegativeBalanceNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:02'!
amount: anAmount
	amount := anAmount.! !

!ScNegativeBalanceNotice methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/9/2012 20:42'!
message
	| source sourceName | 
	sourceName := 'Account'.
	source := account ifNil: [ sourceName := 'Pool'. pool].
	^'{1} {2} has a negative balance!! Balance is: {3}' 
		format: { sourceName. source name. amount printAsCurrency }! !

Object subclass: #ScBasicProfile
	instanceVariableNames: 'pools accounts transactions schedules checkpoint pendingTransactions announcer unallocated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScBasicProfile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/17/2012 21:47'!
generateTestProfile
	^self generateTestProfileAccounts: 5 pools:  50 schedules:  50 transactions: 5000! !

!ScBasicProfile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 10:40'!
generateTestProfileAccounts: accountCount pools: poolCount schedules: scheduleCount transactions: transactionCount
	"Creates a test profile with random data"
	
	|profile intervals collectWithNull |
	collectWithNull := [ :c | (OrderedCollection withAll: c) add: nil; yourself ].
	intervals := {
		ScMonthly every: 1. ScMonthly every: 1. ScMonthly every: 1.  ScMonthly every: 1. ScMonthly every: 1.
		ScMonthly every: 3. ScMonthly every: 3.
		ScYearly every: 1.	
		ScWeekly every: 2. ScWeekly every: 2. 	ScWeekly every: 2. ScWeekly every: 2.
		ScWeekly every: 1}.
	profile := self new.
	"Create some accounts"
	(1 to: accountCount) do: [  :number |
		profile addAccount: (profile newAccount
			name: 'Account ', number printString;
			openingBalance: 1000 atRandom / 100)].
		
	"Create some pools"
	(1 to: poolCount) do: [ :number |
		profile addPool: (profile newPool
			name: 'Pool ', number printString;
			defaultAccount: profile accounts atRandom)].
		
	"Now some schedules"
	(1 to: scheduleCount) do: [ :number |
		| type interval |
		type := ((100 atRandom) < 5) ifTrue: [ #income ] ifFalse: [ #bill ].
		interval := intervals atRandom.
		profile addSchedule: (profile newSchedule
			name: type asString, ' ', number printString;
			interval:  interval class symbol;
			count: interval count;
			start: (Date today subtractDays: (365 atRandom));
			transaction: (type = #income 
				ifTrue: [ profile newIncome
					account: profile accounts atRandom;
					pool: ((100 atRandom < 5) ifTrue: [ profile pools atRandom]);
					amount: (100000 atRandom  / 100) ] 
				ifFalse: [ profile newExpense
					sourceAccount: profile accounts atRandom;
					sourcePool: profile pools atRandom;
					amount: (5000 atRandom /100)]) 
				yourself)].
		
	"Now make sure the bills are covered"
	(profile schedules select: [ :s | s isExpense ]) do: [ :s |
		s coveredBy: (collectWithNull value: (profile schedules select: [ :s2 | s2 isIncome])) atRandom].
	
	"Finally, add some transactions"
	(1 to: transactionCount) do: [ :number | | rand |
		rand := 100 atRandom.
		profile addTransaction: ((
			(rand < 2) ifTrue: [
				profile newIncome
					pool: (collectWithNull value: profile pools) atRandom;
					amount: (500000 atRandom / 100);
					account: profile accounts atRandom] ifFalse: [ 
			(rand < 30) ifTrue: [ 
				profile newExpense 
					sourcePool: profile pools atRandom;
					amount: (50000 atRandom / 100);
					sourceAccount: profile accounts atRandom]
			ifFalse: [ 
				profile newTransaction
					sourcePool: (collectWithNull value: profile pools) atRandom;
					pool: (collectWithNull value: profile pools) atRandom;
					sourceAccount: profile accounts atRandom;
					amount: (50000 atRandom / 100);
					account: profile accounts atRandom]]) 	
			date: (Date today subtractDays: 700 atRandom);
			comment: 'Transaction ', number printString;
			yourself)].
	^profile! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 12/8/2011 18:14'!
accounts
	^accounts! !

!ScBasicProfile methodsFor: 'private' stamp: 'StuartHerring 6/13/2012 20:07'!
add: item to: aCollection
	aCollection add: item.
	item modificationAnnouncer: announcer.
	^item! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:07'!
addAccount: anAccount
	(accounts anySatisfy: [ :p | p name = anAccount name]) 
		ifTrue: [ MbValidationError signal: 'An account with the name ', anAccount name, ' already exists' ].
	self add: anAccount to: accounts.
	self announce: (ScAccountAdded account: anAccount).
	^anAccount! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:17'!
addPool: aPool
	(pools anySatisfy: [ :p | p name = aPool name]) 
		ifTrue: [ MbValidationError signal: 'A pool with the name ', aPool name, ' already exists' ].
	self add: aPool to: pools.
	self announce: (ScPoolAdded pool: aPool).
	^aPool! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:15'!
addSchedule: aSchedule
	(schedules anySatisfy: [ :s | s name = aSchedule name]) 
		ifTrue: [ MbValidationError signal: 'A schedule with the name ', aSchedule name, ' already exists' ].
	self add: aSchedule to: schedules.
	self announce: (ScScheduleAdded schedule: aSchedule).
	^aSchedule.! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:10'!
addTransaction: aTransaction
	self add: aTransaction to: transactions.
	self announce: (ScTransactionAdded transaction: aTransaction).
	^aTransaction! !

!ScBasicProfile methodsFor: 'private' stamp: 'StuartHerring 6/13/2012 22:08'!
announce: anAnnouncement
	^announcer announce: anAnnouncement! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 3/10/2012 08:53'!
checkpointDate
	^checkpoint! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 6/8/2012 17:53'!
expectedTransactionsBy: endDate
	^schedules gather: [ :s | s transactionsBetween: Date today and: endDate]! !

!ScBasicProfile methodsFor: 'testing' stamp: 'StuartHerring 3/11/2012 12:00'!
hasPendingTransactions
	^pendingTransactions notEmpty! !

!ScBasicProfile methodsFor: 'initialization' stamp: 'StuartHerring 6/18/2012 19:59'!
initialize
	super initialize.
	pools := OrderedCollection new.
	accounts := OrderedCollection new.
	transactions := OrderedCollection new.
	schedules := OrderedCollection new.
	checkpoint := Date today.
	pendingTransactions := OrderedCollection new.
	announcer := Announcer new.
	self initializeUnallocated.! !

!ScBasicProfile methodsFor: 'initialization' stamp: 'StuartHerring 6/18/2012 19:59'!
initializeUnallocated
	unallocated := 
		(ScUnallocated profile: self)
			modificationAnnouncer: announcer;
			yourself.
! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 1/30/2012 18:33'!
newAccount
	^ScAccount profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 6/25/2012 21:14'!
newCreditAccount
	^ScCreditAccount profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 2/15/2012 13:03'!
newExpense
	^ScExpense profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 2/15/2012 13:03'!
newIncome
	^ScIncome profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 1/30/2012 18:33'!
newPool
	^ScPool profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 3/4/2012 17:02'!
newSchedule
	^ScSchedule profile: self! !

!ScBasicProfile methodsFor: 'instance creation' stamp: 'StuartHerring 1/30/2012 18:33'!
newTransaction
	^ScTransaction profile: self! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 6/17/2012 20:25'!
notices
	self subclassResponsibility! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 3/10/2012 08:53'!
pendingTransactions
	^pendingTransactions! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 12/11/2011 10:52'!
pools
	^pools! !

!ScBasicProfile methodsFor: 'private' stamp: 'StuartHerring 6/13/2012 20:13'!
remove: item from: aCollection
	aCollection remove: item.
	item modificationAnnouncer: nil.
	^item! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:11'!
removeAccount: anAccount
	self remove: anAccount from: accounts.
	self announce: (ScAccountRemoved account: anAccount).
	^anAccount.! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 7/29/2012 13:21'!
removePendingTransactions: aCollection
	pendingTransactions removeAll: aCollection.
	((aCollection groupedBy: [ :p | p schedule ]) keys) do: [ :s |
		self announce: (ScScheduleModified item: s)]! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:15'!
removePool: aPool
	self remove: aPool from: pools.
	self announce: (ScPoolRemoved pool: aPool).
	^aPool.! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:16'!
removeSchedule: aSchedule
	self remove: aSchedule from: schedules.
	self announce: (ScScheduleRemoved schedule: aSchedule).
	^aSchedule.! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 6/13/2012 22:11'!
removeTransaction: aTransaction
	self remove: aTransaction from: transactions.
	self announce: (ScTransactionRemoved transaction: aTransaction).
	^aTransaction! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 3/4/2012 17:04'!
schedules
	^schedules! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 2/14/2012 16:07'!
transactions
	^transactions! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 5/21/2012 18:48'!
transactionsBetween: startDate and: endDate
	^ transactions select: [ :t | t date between: startDate and: endDate ]! !

!ScBasicProfile methodsFor: 'accessing' stamp: 'StuartHerring 6/18/2012 19:56'!
unallocated
	^unallocated! !

!ScBasicProfile methodsFor: 'actions' stamp: 'StuartHerring 7/29/2012 12:59'!
updatePending
	| today |
	today := Date today.
	schedules do: [ :s | pendingTransactions addAll: ((s futureTransactionsBetween:  checkpoint and: today) collect: [ :t | ScScheduledTransaction schedule: s transaction: t]) ].
	checkpoint := today! !

ScBasicProfile subclass: #ScProfile
	instanceVariableNames: 'analysers schemaVersion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScProfile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/25/2012 21:55'!
schemaVersion
	^7! !

!ScProfile methodsFor: 'initialization' stamp: 'StuartHerring 6/17/2012 20:28'!
initialize
	super initialize.
	self initializeAnalysers.
	schemaVersion := self class schemaVersion.
! !

!ScProfile methodsFor: 'initialization' stamp: 'StuartHerring 6/25/2012 19:45'!
initializeAnalysers
	analysers := { ScAccountsSufficientForBills. ScPoolsNotOverdrawn. ScAccountsNotOverdrawn} 
		collect: [ :a |  (a profile: self) modificationAnnouncer: announcer; yourself ].! !

!ScProfile methodsFor: 'accessing' stamp: 'StuartHerring 6/17/2012 20:25'!
notices
	^analysers gather: [ :each | each notices]! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/25/2012 21:56'!
resetAnnouncer
	announcer := Announcer new.
	{ schedules. transactions. pools. accounts } do: [ :collection |
		collection do: [ :each | each modificationAnnouncer: announcer ]].! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
schemaIsCurrent
	self schemaVersion > self class schemaVersion ifTrue: [ Error signal: 'The database is newer than the code' ].
	^self schemaVersion = self class schemaVersion.! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
schemaVersion
	^schemaVersion ifNil: [0]
	! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
upgradeFromSchemaVersion0
	schedules do: [ :s |
		s instVarNamed: #manualTransactions put: OrderedCollection new].! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
upgradeFromSchemaVersion1
! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
upgradeFromSchemaVersion2
	"Only removed some dead variables"! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
upgradeFromSchemaVersion3
	"Major change in version 3 is the addition of an announcer"
	announcer := Announcer new.
	{ schedules. transactions. pools. accounts } do: [ :collection |
		collection do: [ :each | each modificationAnnouncer: announcer ]]! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:54'!
upgradeFromSchemaVersion4
	"Refresh the announcer again and add analysers"
	announcer := Announcer new.
	{ schedules. transactions. pools. accounts } do: [ :collection |
		collection do: [ :each | each modificationAnnouncer: announcer ]].
	self initializeAnalysers.! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/25/2012 21:56'!
upgradeFromSchemaVersion5
	"Refresh the announcer again"
	self resetAnnouncer.
	"Move time related stuff to a dedicated Timetable instvar"
	schedules do: [ : each | each instVarNamed: #timetable put: (ScTimetable period: (each instVarNamed: #interval) count: (each instVarNamed: #count))].
	
	"Update account and pool balances"
	{accounts. pools} do: [ :collection |
		collection do: [ :each | each updateBalance ]].
	self initializeUnallocated.
	
	"and re-add analysers"
	self initializeAnalysers.
	
! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/25/2012 21:57'!
upgradeFromSchemaVersion6
	"reset announcer again"
	self resetAnnouncer.
	"and re-add analysers"
	self initializeAnalysers.
	
! !

!ScProfile methodsFor: 'schema-migration' stamp: 'StuartHerring 6/17/2012 20:25'!
upgradeSchema
	(self schemaVersion to: (self class schemaVersion - 1)) do: [ :version |
			[ self perform: ('upgradeFromSchemaVersion{1}' format: { version }) asSymbol ]
				on: MagmaTruncationWarning do: [ :w | w resume]].
	schemaVersion := self class schemaVersion.! !

Object subclass: #ScScheduledTransaction
	instanceVariableNames: 'schedule transaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScScheduledTransaction class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 11:16'!
schedule: aSchedule transaction: aTransaction
	^self basicNew
		initializeWithSchedule: aSchedule transaction: aTransaction;
		yourself.! !

!ScScheduledTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 11:15'!
initializeWithSchedule: aSchedule transaction: aTransaction
	self initialize.
	schedule := aSchedule.
	transaction := aTransaction.! !

!ScScheduledTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 11:15'!
schedule
	^schedule! !

!ScScheduledTransaction methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/11/2012 11:15'!
transaction
	^transaction! !

Object subclass: #ScTimetable
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

ScTimetable subclass: #ScDaily
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScDaily class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:48'!
symbol
	^#day! !

!ScDaily methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:32'!
addTo: aDate
	^aDate addDays: count! !

ScTimetable subclass: #ScMonthly
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScMonthly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:39'!
inDays
	^(ScYearly inDays / 12)! !

!ScMonthly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:49'!
symbol
	^#month! !

!ScMonthly methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:34'!
addTo: aDate
	^aDate addMonths: count! !

!ScTimetable class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:37'!
every: count
	^self new
		count: count;
		yourself.! !

!ScTimetable class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:38'!
inDays
	^1! !

!ScTimetable class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/9/2012 14:50'!
period: aSymbol count: aNumber
	^(self subclassForSymbol: aSymbol)
		every: aNumber.! !

!ScTimetable class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:50'!
subclassForSymbol: aSymbol
	^self allSubclasses 
		detect: [ :c | c symbol = aSymbol ]
		ifNone: [ Error signal: 'Unknown symbol: ', aSymbol ]! !

!ScTimetable class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:48'!
symbol
	self subclassResponsibility ! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:31'!
addTo: aDate
	self subclassResponsibility.! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:36'!
count
	^count! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:34'!
count: aCount
	count := aCount asInteger! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:41'!
inDays
	^self class inDays * count! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 18:03'!
initialize
	super initialize.
	count := 1.! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 5/21/2012 21:01'!
ratioFor: aTimetable
	"Returns the multiplier required to turn the receiver into an equivalent length of time as that represented by the argument"
	^aTimetable inDays / self inDays ! !

!ScTimetable methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:24'!
symbol
	^self class symbol! !

ScTimetable subclass: #ScWeekly
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScWeekly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/17/2012 17:39'!
inDays
	^7! !

!ScWeekly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:49'!
symbol
	^#week! !

!ScWeekly methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:32'!
addTo: aDate
	^aDate addDays: (count * 7)! !

ScTimetable subclass: #ScYearly
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Smallchange-Core'!

!ScYearly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 3/25/2012 13:23'!
inDays
	"Roughly account for leap years"
"	^(1461/4)"
	"Or not"
	^365! !

!ScYearly class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/11/2012 09:49'!
symbol
	^#year! !

!ScYearly methodsFor: 'as yet unclassified' stamp: 'StuartHerring 6/18/2012 18:34'!
addTo: aDate
	^aDate addMonths: 12 * count! !
